@@ -21,18 +21,12 @@
  
 # deps: python-html python-image python-netclient python-misc python-pygtk python-mime python-json
 
-# todo:
-# save last viewed cache
-# notizspeichern prüfen
-
-
  
 ### For the gui :-)
 
-
-# todo
-# 
-import math
+from math import ceil
+from os import path, system
+import re
 
 import coordfinder
 import geo
@@ -40,10 +34,8 @@ import geocaching
 import gtk
 import hildon
 import openstreetmap
-import os
 import pango
-import re
-from simplegui import SimpleGui
+from simplegui import SimpleGui, UpdownRows
 
 class HildonGui(SimpleGui):
 
@@ -73,8 +65,8 @@ class HildonGui(SimpleGui):
     def __init__(self, core, pointprovider, userpointprovider, dataroot):
         gtk.gdk.threads_init()
         self.ts = openstreetmap.TileServer()
-        self.noimage_cantload = gtk.gdk.pixbuf_new_from_file(os.path.join(dataroot, 'noimage-cantload.png'))
-        self.noimage_loading = gtk.gdk.pixbuf_new_from_file(os.path.join(dataroot, 'noimage-loading.png'))
+        self.noimage_cantload = gtk.gdk.pixbuf_new_from_file(path.join(dataroot, 'noimage-cantload.png'))
+        self.noimage_loading = gtk.gdk.pixbuf_new_from_file(path.join(dataroot, 'noimage-loading.png'))
         
         self.core = core
         self.core.connect('map-changed', self._on_map_changed)
@@ -82,7 +74,6 @@ class HildonGui(SimpleGui):
         self.core.connect('fieldnotes-changed', self._on_fieldnotes_changed)
 
         self.pointprovider = pointprovider
-        self.userpointprovider = userpointprovider
 
         self.build_tile_loaders()
                 
@@ -95,11 +86,13 @@ class HildonGui(SimpleGui):
         self.current_target = None
         self.gps_data = None
         self.gps_has_fix = False
-        self.gps_last_position = (0, 0)
+        self.gps_last_good_fix = None
+        self.gps_last_screen_position = (0, 0)
         self.banner = None
         self.old_cache_window = None
         self.old_search_window = None
         self.cache_calc_vars = {}
+        #self.osd_string = ''
                 
         self.dragging = False
         self.block_changes = False
@@ -110,6 +103,7 @@ class HildonGui(SimpleGui):
         self.drag_offset_y = 0
         self.notes_changed = False
         self.map_center_x, self.map_center_y = 100, 100
+        self.active_tile_loaders = []
         
         gtk.set_application_name("Geocaching Tool")
         program = hildon.Program.get_instance()
@@ -121,6 +115,30 @@ class HildonGui(SimpleGui):
         self.window.set_app_menu(self._create_main_menu())
         self.update_fieldnotes_display()
 
+        gtk.link_button_set_uri_hook(self._open_browser)
+        #self.show_coordinate_input(geo.Coordinate(49.344, 6.584))
+
+    def _open_browser(self, widget, link):
+        system("browser --url='%s' &" % link)
+
+    def show_coordinate_input(self, start):
+        udr = UpdownRows(self.format, start, True)
+        dialog = gtk.Dialog("Edit Target", None, gtk.DIALOG_MODAL, ("OK", gtk.RESPONSE_ACCEPT))
+        dialog.set_size_request(-1, 480)
+        dialog.vbox.pack_start(udr.table_lat)
+                
+        dialog.vbox.pack_start(udr.table_lon)
+                
+        dialog.show_all()
+        res = dialog.run()
+        dialog.destroy()
+        if res == gtk.RESPONSE_ACCEPT:
+            c = udr.get_value()
+            c.name = 'manual'
+            return c
+        else:
+            return startd
+
     def set_tile_loader(self, widget, loader):
         if widget.get_active():
             self.tile_loader = loader
@@ -130,7 +148,7 @@ class HildonGui(SimpleGui):
     def _on_key_press(self, window, event):
         return
         if event.keyval == gtk.keysyms.F7:
-            self.zoom( + 1)
+            self.zoom(+ 1)
             return False
         elif event.keyval == gtk.keysyms.F8:
             self.zoom(-1)
@@ -140,7 +158,7 @@ class HildonGui(SimpleGui):
     def _create_main_view(self):
         root = gtk.VBox()
 
-        self.main_gpspage = gtk.Table(7, 2)
+        self.main_gpspage = gtk.Table(7, 3)
         self.drawing_area_arrow = gtk.DrawingArea()
 
         self.label_dist = gtk.Label()
@@ -170,30 +188,31 @@ class HildonGui(SimpleGui):
         button.set_size_request(270, -1)
         self.label_target = button
 
-        buttons = gtk.HBox()
+        #buttons = gtk.HBox()
 
-        button = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
-        button.set_title("Details")
-        button.set_sensitive(False)
-        button.connect("clicked", self._on_show_cache_details, None)
-        buttons.pack_start(button, True, True)
+        button_details = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        button_details.set_title("Details")
+        button_details.set_sensitive(False)
+        button_details.connect("clicked", self._on_show_cache_details, None)
+        #buttons.pack_start(button, True, True)
         
-        self.button_show_details_small = button
+        self.button_show_details_small = button_details
         
-        button = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
-        button.set_label("Map")
-        button.connect("clicked", self._on_set_active_page, False)
-        buttons.pack_start(button, True, True)
+        button_map = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        button_map.set_label("Map")
+        button_map.connect("clicked", self._on_set_active_page, False)
+        #buttons.pack_start(button, True, True)
 
 
-        self.main_gpspage.attach(self.label_dist, 1, 2, 0, 1, gtk.FILL, gtk.FILL | gtk.EXPAND)
-        self.main_gpspage.attach(self.label_bearing, 1, 2, 1, 2, gtk.FILL, gtk.FILL | gtk.EXPAND)
-        self.main_gpspage.attach(self.label_altitude, 1, 2, 2, 3, gtk.FILL, gtk.FILL | gtk.EXPAND)
-        self.main_gpspage.attach(self.label_latlon, 1, 2, 3, 4, gtk.FILL, gtk.FILL | gtk.EXPAND)
-        self.main_gpspage.attach(self.label_quality, 1, 2, 4, 5, gtk.FILL, gtk.FILL | gtk.EXPAND)
-        self.main_gpspage.attach(self.label_target, 1, 2, 5, 6, gtk.FILL, gtk.FILL | gtk.EXPAND)
-        self.main_gpspage.attach(buttons, 1, 2, 6, 7, gtk.FILL, gtk.FILL | gtk.EXPAND)
-        self.main_gpspage.attach(self.drawing_area_arrow, 0, 1, 0, 7, gtk.EXPAND | gtk.FILL, gtk.EXPAND | gtk.FILL)
+        self.main_gpspage.attach(self.label_dist, 1, 3, 0, 1, gtk.FILL, gtk.FILL | gtk.EXPAND)
+        self.main_gpspage.attach(self.label_altitude, 1, 2, 1, 2, gtk.FILL, gtk.FILL | gtk.EXPAND)
+        self.main_gpspage.attach(self.label_bearing, 2, 3, 1, 2, gtk.FILL, gtk.FILL | gtk.EXPAND)
+        self.main_gpspage.attach(self.label_latlon, 1, 3, 2, 3, gtk.FILL, gtk.FILL | gtk.EXPAND)
+        self.main_gpspage.attach(self.label_quality, 1, 3, 3, 4, gtk.FILL, gtk.FILL | gtk.EXPAND)
+        self.main_gpspage.attach(self.label_target, 1, 3, 4, 5, gtk.FILL, 0)
+        self.main_gpspage.attach(button_details, 1, 2, 5, 6, gtk.FILL, 0)
+        self.main_gpspage.attach(button_map, 2, 3, 5, 6, gtk.FILL, 0)
+        self.main_gpspage.attach(self.drawing_area_arrow, 0, 1, 0, 6, gtk.EXPAND | gtk.FILL, gtk.EXPAND | gtk.FILL)
 
         self.main_mappage = gtk.VBox()
         self.drawing_area = gtk.DrawingArea()
@@ -206,16 +225,15 @@ class HildonGui(SimpleGui):
         self.button_track = button
         buttons.pack_start(button, True, True)
 
-
+        icon = gtk.image_new_from_stock(gtk.STOCK_ZOOM_IN, gtk.ICON_SIZE_BUTTON)
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        button.set_label("+")
+        button.set_image(icon)
         button.connect("clicked", self.on_zoomin_clicked, None)
         buttons.pack_start(button, True, True)
 
-
-
+        icon = gtk.image_new_from_stock(gtk.STOCK_ZOOM_OUT, gtk.ICON_SIZE_BUTTON)
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        button.set_label("-")
+        button.set_image(icon)
         button.connect("clicked", self.on_zoomout_clicked, None)
         buttons.pack_start(button, True, True)
 
@@ -357,7 +375,7 @@ class HildonGui(SimpleGui):
         sel_size.append_text('huge')
         sel_size.append_text('other')
         sel_size.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_MULTIPLE)
-        pick_size = hildon.PickerButton(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        pick_size = hildon.PickerButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
         pick_size.set_selector(sel_size)
         pick_size.set_title("Select Size(s)")
         for i in range(5):
@@ -372,7 +390,7 @@ class HildonGui(SimpleGui):
         sel_type.append_text('mystery')
         sel_type.append_text('all')
         sel_type.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_MULTIPLE)
-        pick_type = hildon.PickerButton(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        pick_type = hildon.PickerButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
         pick_type.set_selector(sel_type)
         pick_type.set_title("Select Type(s)")
         sel_type.unselect_all(0)
@@ -384,7 +402,7 @@ class HildonGui(SimpleGui):
         sel_status.append_text('found')
         sel_status.append_text('marked')
         sel_status.append_text('not found & marked')
-        pick_status = hildon.PickerButton(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        pick_status = hildon.PickerButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
         pick_status.set_selector(sel_status)
         pick_status.set_title("Select Status")
         
@@ -396,7 +414,7 @@ class HildonGui(SimpleGui):
         sel_diff.append_text('3..4')
         sel_diff.append_text('4.5..5')
         sel_diff.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_MULTIPLE)
-        pick_diff = hildon.PickerButton(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        pick_diff = hildon.PickerButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
         pick_diff.set_selector(sel_diff)
         pick_diff.set_title("Select Difficulty")
         for i in range(3):
@@ -407,13 +425,13 @@ class HildonGui(SimpleGui):
         sel_terr.append_text('3..4')
         sel_terr.append_text('4.5..5')
         sel_terr.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_MULTIPLE)
-        pick_terr = hildon.PickerButton(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        pick_terr = hildon.PickerButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
         pick_terr.set_selector(sel_terr)
         pick_terr.set_title("Select Terrain")
         for i in range(3):
             sel_terr.select_iter(0, sel_terr.get_model(0).get_iter(i), False)
         
-        name = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+        name = hildon.Entry(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
         name.set_placeholder("search for name...")
         
         t = gtk.Table(4, 3, False)
@@ -510,7 +528,6 @@ class HildonGui(SimpleGui):
         if self.gps_data != None and self.gps_data.position != None:
             for c in caches:
                 c.prox = c.distance_to(self.gps_data.position)
-                print c.difficulty
         else:
             for c in caches:
                 c.prox = None
@@ -523,7 +540,7 @@ class HildonGui(SimpleGui):
         col1 = tv.append_column(ls, gtk.CellRendererText())
         
         c1cr = gtk.CellRendererText()
-        c1cr.ellipsize=pango.ELLIPSIZE_MIDDLE
+        c1cr.ellipsize = pango.ELLIPSIZE_MIDDLE
         c2cr = gtk.CellRendererText()
         c3cr = gtk.CellRendererText()
         c4cr = gtk.CellRendererText()
@@ -539,7 +556,7 @@ class HildonGui(SimpleGui):
         col1.set_attributes(c4cr, text=3)
 
         def select_cache(widget, data, more):
-            self.show_cache(self._get_selected(widget)[4])
+            self.show_cache(self._get_selected(tv)[4])
         
         tv.connect("changed", select_cache, None)
 
@@ -549,7 +566,7 @@ class HildonGui(SimpleGui):
             ls.clear()
             caches.sort(cmp=sortfunc)
             for c in caches:
-                ls.append([self.shorten_name(c.title, 40), " " + c.get_size_string(), ' D%s T%s' % (c.get_difficulty(), c.get_terrain()), " " + self.__format_distance(c.prox),c])
+                ls.append([self.shorten_name(c.title, 40), " " + c.get_size_string(), ' D%s T%s' % (c.get_difficulty(), c.get_terrain()), " " + self.__format_distance(c.prox), c])
             tv.handler_unblock_by_func(select_cache)
 
 
@@ -589,7 +606,7 @@ class HildonGui(SimpleGui):
     def _on_show_options(self, widget, data):
         dialog = gtk.Dialog("options", None, gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
         
-        opts = gtk.Table(5, 2)
+        opts = gtk.Table(6, 2)
         opts.attach(gtk.Label("Username"), 0, 1, 0, 1)
         username = hildon.Entry(gtk.HILDON_SIZE_AUTO)
         #username.set_property(hildon.HILDON_AUTOCAP, False)
@@ -603,16 +620,26 @@ class HildonGui(SimpleGui):
         opts.attach(password, 1, 2, 1, 2)
         password.set_text(self.settings['options_password'])
         
-        check_dl_images = gtk.CheckButton("Don't Download Images")
+        check_dl_images = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        check_dl_images.set_label("Don't Download Images")
         check_dl_images.set_active(self.settings['download_noimages'])
-        check_show_cache_id = gtk.CheckButton("Show Geocache ID on Map")
+
+        check_show_cache_id = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        check_show_cache_id.set_label("Show Geocache ID on Map")
         check_show_cache_id.set_active(self.settings['options_show_name'])
-        check_hide_found = gtk.CheckButton("Hide Found Geocaches on Map")
+
+        check_map_double_size = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        check_map_double_size.set_label("Show Map in double size (ugly)")
+        check_map_double_size.set_active(self.settings['options_map_double_size'])
+
+        check_hide_found = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        check_hide_found.set_label("Hide Found Geocaches on Map")
         check_hide_found.set_active(self.settings['options_hide_found'])
         
         opts.attach(check_dl_images, 0, 2, 2, 3)
         opts.attach(check_show_cache_id, 0, 2, 3, 4)
-        opts.attach(check_hide_found, 0, 2, 4, 5)
+        opts.attach(check_map_double_size, 0, 2, 4, 5)
+        opts.attach(check_hide_found, 0, 2, 5, 6)
         
         dialog.vbox.pack_start(opts)
         
@@ -625,6 +652,7 @@ class HildonGui(SimpleGui):
             self.settings['options_password'] = password.get_text()
             self.settings['download_noimages'] = check_dl_images.get_active()
             self.settings['options_show_name'] = check_show_cache_id.get_active()
+            self.settings['options_map_double_size'] = check_map_double_size.get_active()
             self.settings['options_hide_found'] = check_hide_found.get_active()
             self.core.on_userdata_changed(self.settings['options_username'], self.settings['options_password'])
 
@@ -635,27 +663,39 @@ class HildonGui(SimpleGui):
             c = self.gps_data.position
         else:
             c = geo.Coordinate(0, 0)
+
         dialog = gtk.Dialog("change target", None, gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
-        h_lat = gtk.HBox()      
-        l_lat = gtk.Label("Lat/Lon: ")
-        e_lat = hildon.Entry(gtk.HILDON_SIZE_AUTO)
-        e_lat.set_property('hildon-input-mode', gtk.HILDON_GTK_INPUT_MODE_ALPHA | gtk.HILDON_GTK_INPUT_MODE_SPECIAL | gtk.HILDON_GTK_INPUT_MODE_TELE)
-        e_lat.set_text("%s %s" % (c.get_lat(self.format), c.get_lon(self.format)))
-        h_lat.pack_start(l_lat, True)
-        h_lat.pack_start(e_lat, True)
-        
-        #e_lat.set_property("input-mode", gtk.HILDON_INPUT_MODE_HINT_NUMERICSPECIAL)
-        #l = gtk.Label("Use + for N/E, - for W/S.")
-        #l.set_alignment(0, 0.5)
-        #dialog.vbox.pack_start(l, False)
-        dialog.vbox.pack_start(h_lat, False)
+
+        bar_label = gtk.Label("Lat/Lon: ")
+        bar_entry = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+        bar_entry.set_property('hildon-input-mode', gtk.HILDON_GTK_INPUT_MODE_ALPHA | gtk.HILDON_GTK_INPUT_MODE_SPECIAL | gtk.HILDON_GTK_INPUT_MODE_TELE)
+        bar_entry.set_text(c.get_latlon(self.format))
+
+        def show_coord_input(widget):
+            try:
+                m = geo.try_parse_coordinate(bar_entry.get_text())
+            except Exception, e:
+                m = c
+            m_new = self.show_coordinate_input(m)
+            bar_entry.set_text(m_new.get_latlon(self.format))
+
+        bar_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        bar_button.set_label(" Edit ")
+        bar_button.connect("clicked", show_coord_input)
+
+        bar = gtk.HBox()      
+        bar.pack_start(bar_label, False)
+        bar.pack_start(bar_entry, True)
+        bar.pack_start(bar_button, False)
+
+        dialog.vbox.pack_start(bar, False)
 
         def sel_coord(widget, data, clist):
             tm = widget.get_model(0)
             iter = tm.get_iter(0)
             widget.get_selected(0, iter)
             coord = clist[tm.get_path(iter)[0]]
-            e_lat.set_text("%s %s" % (coord.get_lat(self.format), coord.get_lon(self.format)))
+            bar_entry.set_text(coord.get_latlon(self.format))
 
         if self.current_cache != None:
             sel, clist = self._get_coord_selector(self.current_cache, sel_coord, True)
@@ -667,7 +707,7 @@ class HildonGui(SimpleGui):
         result = dialog.run()
         dialog.hide()
         if result == gtk.RESPONSE_ACCEPT:
-            self.set_target(geo.try_parse_coordinate(e_lat.get_text()))
+            self.set_target(geo.try_parse_coordinate(bar_entry.get_text()))
 
     def show_cache(self, cache, select=True):
         if cache == None:
@@ -717,10 +757,12 @@ class HildonGui(SimpleGui):
         l.set_markup("<b>Open Website</b>")
         l.set_alignment(0, 0.5)
         p.attach(l, 0, 1, 8, 9)
-        #z = gtk.HBox(True)
-        #z.pack_start(gtk.LinkButton("http://www.geocaching.com/seek/cache_details.aspx?wp=%s" % cache.name, 'Listing'))
-        #z.pack_start(gtk.LinkButton("http://www.geocaching.com/seek/log.aspx?wp=%s" % cache.name, 'Log'))
-        #p.attach(z, 1, 2, 8, 9)
+        z = gtk.HBox(True)
+
+        z.pack_start(gtk.LinkButton("http://www.geocaching.com/seek/cache_details.aspx?wp=%s" % cache.name, 'Listing'))
+        z.pack_start(gtk.LinkButton("http://www.geocaching.com/seek/log.aspx?wp=%s" % cache.name, 'Post Log'))
+        z.pack_start(gtk.LinkButton("http://www.geocaching.com/seek/cache_details.aspx?wp=%s&log=y#ctl00_ContentBody_CacheLogs" % cache.name, 'All Logs'))
+        p.attach(z, 1, 2, 8, 9)
 
         # cache-was-not-downloaded-yet-warning
         if not cache.was_downloaded():
@@ -815,19 +857,38 @@ class HildonGui(SimpleGui):
         notebook.connect("switch-page", switchpage)
 
         # notes
-        p = hildon.PannableArea()
+        pan = hildon.PannableArea()
         self.cache_notes = gtk.TextView()
         #cache_notes.set_editable(True)
         self.cache_notes.get_buffer().set_text(cache.notes)
-        p.add(self.cache_notes)
+        pan.add(self.cache_notes)
 
-        notebook.append_page(p, gtk.Label("notes"))
         self.notes_changed = False
         def notes_changed(caller):
             self.notes_changed = True
-
         self.cache_notes.get_buffer().connect('changed', notes_changed)
 
+        def add_waypoint(widget):
+            if self.gps_data != None and self.gps_data.position != None:
+                c = self.gps_data.position
+            elif self.current_target != None:
+                c = self.current_target
+            else:
+                c = geo.Coordinate(0, 0)
+            res = self.show_coordinate_input(c)
+            text = "\n%s\n" % res.get_latlon(self.format)
+            self.cache_notes.get_buffer().insert(self.cache_notes.get_buffer().get_end_iter(), text)
+                
+
+        button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        button.set_label("Add Waypoint")
+        button.connect("clicked", add_waypoint)
+
+        p = gtk.VBox()
+        p.pack_start(pan, True)
+        p.pack_start(button, False)
+        notebook.append_page(p, gtk.Label("notes"))
+
 
         win.add(notebook)
         
@@ -872,7 +933,6 @@ class HildonGui(SimpleGui):
         self.current_cache_window_open = True
 
     def build_coordinates(self, cache, p):
-        print 'updating'
         widget_coords, clist = self._get_coord_selector(cache, lambda x, y, z: True)
 
 
@@ -896,13 +956,13 @@ class HildonGui(SimpleGui):
                 return
             self.core.set_alternative_position(cache, c)
             
-        h = gtk.HBox()
-        button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+        h = gtk.HBox(True)
+        button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
         button.set_label("set as target")
         button.connect("clicked", set_coord_as_target)
         h.pack_start(button)
 
-        button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+        button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
         button.set_label("set as main coordinate")
         button.connect("clicked", set_alternative_position)
         h.pack_start(button)
@@ -956,7 +1016,7 @@ class HildonGui(SimpleGui):
         count = len(self.cache_calc.requires)
         # create table with n columns.
         cols = 7
-        rows = int(math.ceil(float(count) / float(cols)))
+        rows = int(ceil(float(count) / float(cols)))
         table = gtk.Table(rows, cols)
         i = 0
         requires_sort = list(self.cache_calc.requires)
@@ -1022,10 +1082,10 @@ class HildonGui(SimpleGui):
         self.cache_calc_viewport.show_all()
 
         
-    def _on_show_image(self, path, caption):
-        fullpath = os.path.join(self.settings['download_output_dir'], path)
-        if not os.path.exists(fullpath):
-            print "ex nicht: " + fullpath
+    def _on_show_image(self, dpath, caption):
+        fullpath = path.join(self.settings['download_output_dir'], dpath)
+        if not path.exists(fullpath):
+            print "file does not exist: " + fullpath
             return
         win = hildon.StackableWindow()
         win.set_title(caption)
@@ -1047,13 +1107,13 @@ class HildonGui(SimpleGui):
         breaks are posix newlines (\n).
         """
         return reduce(lambda line, word, width=width: '%s%s%s' %
-                  (line,
-                   ' \n'[(len(line)-line.rfind('\n')-1
-                         + len(word.split('\n',1)[0]
-                              ) >= width)],
-                   word),
-                  text.split(' ')
-                 )
+                      (line,
+                      ' \n'[(len(line)-line.rfind('\n')-1
+                      + len(word.split('\n', 1)[0]
+                      ) >= width)],
+                      word),
+                      text.split(' ')
+                      )
 
     def _get_coord_selector(self, cache, callback, no_empty=False):
         selector = hildon.TouchSelector(text=True)
@@ -1124,30 +1184,6 @@ class HildonGui(SimpleGui):
         self.hide_cache_view(None, None)
 
 
-    @staticmethod
-    def shorten_name(s, chars):
-        max_pos = chars
-        min_pos = chars - 10
-
-        NOT_FOUND = -1
-
-        suffix = '…'
-
-        # Case 1: Return string if it is shorter (or equal to) than the limit
-        length = len(s)
-        if length <= max_pos:
-            return s
-        else:
-            # Case 2: Return it to nearest period if possible
-            try:
-                end = s.rindex('.', min_pos, max_pos)
-            except ValueError:
-                # Case 3: Return string to nearest space
-                end = s.rfind(' ', min_pos, max_pos)
-                if end == NOT_FOUND:
-                    end = max_pos
-            return s[0:end] + suffix
-
 
     @staticmethod
     def _get_selected(widget):
@@ -1175,12 +1211,8 @@ class HildonGui(SimpleGui):
             self.show_cache(cache)
             return False
         else:
-            print self.current_cache, cache.name, self.current_cache_window_open
             return False
 
-    def _on_map_changed(self, something):
-        self.redraw_marks()
-        return False
 
 
 
@@ -1215,14 +1247,18 @@ class HildonGui(SimpleGui):
         fieldnotes.set_placeholder("Your fieldnote text...")
         fieldnotes.get_buffer().set_text(cache.fieldnotes)
 
-        fieldnotes_log_as = gtk.combo_box_new_text()
+        fieldnotes_log_as_selector = hildon.TouchSelector(text=True)
+        
         for text, status in statuses:
-            fieldnotes_log_as.append_text(text)
+            fieldnotes_log_as_selector.append_text(text)
         i = 0
         for text, status in statuses:
             if cache.log_as == status:
-                fieldnotes_log_as.set_active(i)
+                fieldnotes_log_as_selector.select_iter(0, fieldnotes_log_as_selector.get_model(0).get_iter(i), False)
             i += 1
+        fieldnotes_log_as = hildon.PickerButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+        fieldnotes_log_as.set_title('Log Type')
+        fieldnotes_log_as.set_selector(fieldnotes_log_as_selector)
 
         dialog.vbox.pack_start(fieldnotes_log_as, False)
         dialog.vbox.pack_start(fieldnotes, True)
@@ -1230,17 +1266,17 @@ class HildonGui(SimpleGui):
         result = dialog.run()
         dialog.hide()
         if result != gtk.RESPONSE_ACCEPT:
-            print 'not logged'
+            print 'Not logging this fieldnote'
             return
         from time import gmtime
         from time import strftime
-        
-        cache.logas = statuses[fieldnotes_log_as.get_active()][1]
-        cache.logdate = strftime('%Y-%m-%d', gmtime())
+
+        cache.log_as = statuses[fieldnotes_log_as_selector.get_selected_rows(0)[0][0]][1]
+        cache.log_date = strftime('%Y-%m-%d', gmtime())
         cache.fieldnotes = fieldnotes.get_buffer().get_text(fieldnotes.get_buffer().get_start_iter(), fieldnotes.get_buffer().get_end_iter())
-        self.core.write_fieldnote(self.current_cache, cache.logas, cache.logdate, cache.fieldnotes)
-
-
+        self.core.write_fieldnote(self.current_cache, cache.log_as, cache.log_date, cache.fieldnotes)
+        
+        
     def _on_upload_fieldnotes(self, some, thing):
         self.core.on_upload_fieldnotes()
 
@@ -1273,7 +1309,7 @@ class HildonGui(SimpleGui):
 
     def set_center(self, coord, noupdate=False):
         SimpleGui.set_center(self, coord, noupdate)
-        self.button_center_as_target.set_value("%s %s" % (coord.get_lat(self.format), coord.get_lon(self.format)))
+        self.button_center_as_target.set_value(coord.get_latlon(self.format))
 
     def set_current_cache(self, cache):
         self.current_cache = cache
@@ -1288,7 +1324,7 @@ class HildonGui(SimpleGui):
 
     def set_target(self, cache):
         self.current_target = cache
-        coord = "%s %s" % (cache.get_lat(self.format), cache.get_lon(self.format))
+        coord = cache.get_latlon(self.format)
         self.label_target.set_value(coord)
         self.button_goto_target.set_value(coord)
         
@@ -1319,8 +1355,7 @@ class HildonGui(SimpleGui):
     def _drag_end(self, widget, event):
         SimpleGui._drag_end(self, widget, event)
         c = self.ts.num2deg(self.map_center_x, self.map_center_y)
-        self.button_center_as_target.set_value("%s %s" % (c.get_lat(self.format), c.get_lon(self.format)))
-
+        self.button_center_as_target.set_value(c.get_latlon(self.format))
 
     def on_zoom_changed(self, blub):
         self.zoom()
@@ -1409,6 +1444,7 @@ class HildonGui(SimpleGui):
 
     def update_gps_display(self):
         if self.gps_data == None:
+            #self.osd_string = "<span gravity='west' size='xx-large'>No Fix </span>"
             return
 
         if self.gps_data.sats == 0:
@@ -1417,18 +1453,27 @@ class HildonGui(SimpleGui):
             text = "%d/%d sats, error: ±%3.1fm" % (self.gps_data.sats, self.gps_data.sats_known, self.gps_data.error)
         self.label_quality.set_markup("Accuracy\n<small>%s</small>" % text)
         if self.gps_data.altitude == None or self.gps_data.bearing == None:
+            #self.osd_string = "<span gravity='west' size='xx-large'>No Fix </span>"
             return
 
         self.label_altitude.set_markup("Altitude\n<small>%d m</small>" % self.gps_data.altitude)
         self.label_bearing.set_markup("Bearing\n<small>%d°</small>" % self.gps_data.bearing)
-        self.label_latlon.set_markup("Current Position\n<small>%s %s</small>" % (self.gps_data.position.get_lat(self.format), self.gps_data.position.get_lon(self.format)))
+        self.label_latlon.set_markup("Current Position\n<small>%s</small>" % self.gps_data.position.get_latlon(self.format))
 
         if self.current_target == None:
+            #self.osd_string = ""
             return
 
         target_distance = self.gps_data.position.distance_to(self.current_target)
-        
-        self.label_dist.set_markup("Distance\n<small>%s</small>" % self.__format_distance(target_distance))
+        td_string = self.__format_distance(target_distance)
+        if self.gps_has_fix:
+            self.label_dist.set_markup("<span size='xx-large'>%s</span>" % td_string)
+
+            #self.osd_string = "<span gravity='west' size='xx-large'>%s </span>" % td_string
+        else:
+            self.label_dist.set_markup("<span size='xx-large'>No Fix</span>")
+
+            #self.osd_string = "<span gravity='west' size='xx-large'>No Fix </span>"
 
     def __format_distance(self, distance):
         if distance == None:
